/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2012, 2015 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_main.c
* Version      : Code Generator for RL78/G10 V1.04.02.01 [15 May 2015]
* Device(s)    : R5F10Y46
* Tool-Chain   : CCRL
* Description  : This file implements main function.
* Creation Date: 2015/09/14
* Modified Date: 2015/10/06 unify interface
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_cgc.h"
#include "r_cg_port.h"
#include "r_cg_tau.h"
#include "r_cg_wdt.h"
#include "r_cg_sau.h"
/* Start user code for include. Do not edit comment generated here */
#include "r_IIC_LIB.h"
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
#pragma interrupt INTWDTI r_it_interrupt
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
volatile uint8_t    g_write_commmand[2];
volatile uint8_t    g_write_data[2];

volatile uint8_t   g_05s_status;        /* 0.5s interrupt flag          */
volatile uint8_t   g_05s_count;         /* 0.5s counter                 */

#define     slaveaddr   (0xA0)          /* IIC bus slave address        */

/*----------------------------------------------

    instruction code for LCD

-----------------------------------------------*/

#define     COMBYTE     (0x00)        	        /* control byte for command     */
#define     DATABYTE    (0x80)        	        /* control byte for data        */
#define     CLRDISP     (0x01)        	        /* clear display instruction    */
#define     HOMEPOSI    (0x02)        	        /* home position instruction    */
#define     LCD_Mode    (0b00111000)            /* operation mode instruction   */
/*                         ++||||------ must be 0
                             +|||------ must be 1
                              ||+------ 5*10/5*8dots
                              |+------- 2line/1line
                              +-------- data lengt 8bit(FIX)
*/
//#define     DISPON      (0b00001111)            /* display on instruction       */
#define     DISPON      (0b00001100)            /* display on instruction       */
/*                         ++++||||---- must be 0
                               +|||---- must be 1
                                ||+---- blinking of cursor
                                |+----- cursor
                                +------ display
*/
#define     ENTRY_Mode  (0b00000110)            /* entry mode instruction       */
/*                         +++++|||---- must be 0
                                +||---- must be 1
                                 |+---- shift
                                 +----- shift direction
*/

static const uint8_t dispdata1[] = {"Renesas RL78/G10"};

void init_LCD(void);
MD_STATUS  set_data(uint8_t datacode);
MD_STATUS  set_command(uint8_t comcode);

void wait_100ms(void);
void wait_500ms(void);

/* End user code. Do not edit comment generated here */

void R_MAIN_UserInit(void);
/***********************************************************************************************************************
* Function Name: main
* Description  : This function implements main function.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void main(void)
{
    R_MAIN_UserInit();
    /* Start user code. Do not edit comment generated here */

  {
    int8_t CPNT1,CPNT2;

    CPNT1 = 0;

    while (1U)
    {

        wait_500ms();                           /* wait for 0.5s flag       */


        for ( CPNT1 = 0 ; CPNT1 < 16 ; CPNT1++ )
        {

            set_command(0b10001111);            /* move end of line 1       */

            set_data(dispdata1[CPNT1]);         /* get 1st data             */

            for ( CPNT2 = 14 ; CPNT2+1 > CPNT1 ; CPNT2-- )
            {
                wait_100ms();                   /* wait for 100ms           */
                set_command(0x80|CPNT2);        /* move end of line 1       */
                set_data(dispdata1[CPNT1]);     /* move character           */
                set_data(0x20);                 /* clear old data           */

            }
            wait_500ms();                       /* wait for 500ms           */
        }

     }
  }
    /* End user code. Do not edit comment generated here */
}
/***********************************************************************************************************************
* Function Name: R_MAIN_UserInit
* Description  : This function adds user code before implementing main function.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_MAIN_UserInit(void)
{
    /* Start user code. Do not edit comment generated here */

    uint8_t CNT;

/* use TM01 for delay count mode */

    IIC_TM01_init();                    /* enable TM01 operation    */

/* start IT                       */

    g_05s_status = 0;
    g_05s_count = 0;
//    WDTIIF = 0;                         /* clear interrupt flag     */
//    WDTIMK = 0;                         /* enable WDTI interrupt    */
    WDTE = 0xAC;                        /* clear WDT                */

    EI();

    wait_500ms();                       /* wait for 0.5s	    */

/*--------------------------------------
    LCD initialize
----------------------------------------*/


    init_LCD();                         /* initialize LCD           */

/* display 1st line */

    for ( CNT = 0 ; CNT < 16 ; CNT++ )
    {
        set_data(dispdata1[CNT]);

    }

    /* End user code. Do not edit comment generated here */
}

/* Start user code for adding. Do not edit comment generated here */
/*******************************************************************************
* Function Name: init_LCD
* Description  : This function initialize LCD
* Arguments    : none
* Return Value : none
********************************************************************************/
void init_LCD(void)
{
    set_command(CLRDISP);               /* initialize LCD display       */
    wait_time(240);                     /* wait for 20us*240=4.8ms      */
    set_command(LCD_Mode);              /* set LCD operation mode       */
    set_command(DISPON);                /* display on                   */
    set_command(ENTRY_Mode);            /* set data entry mode          */
}

/*******************************************************************************
* Function Name: set_data
* Description  : This function send data to LCD
* Arguments    : data code
* Return Value : result
                0x00 : success
                other : error
********************************************************************************/
MD_STATUS set_data(uint8_t datacode)
{

    MD_STATUS status;

    g_write_data[0] = DATABYTE;         /* set command for LCD          */
    g_write_data[1] = datacode;         /* set data for LCD             */

/*
    send command instruction to LCD
*/
    status = R_IIC_Master_Send( slaveaddr , (uint8_t *__near)g_write_data , 2 );
    if( status == 0)
    {

/*
    wait for transmission end
*/
        do
        {
            status = R_IIC_check_comstate();
        }while(( status & 0x0F ) == 0x01 );

/*
    release IIC  bus
*/
        R_IIC_StopCondition();          /* issue stop condition         */

        wait_time(3);                   /* wait for exacution time      */
    }
    return (status);

}

/*******************************************************************************
* Function Name: set_command
* Description  : This function send command to LCD
* Arguments    : command code
* Return Value : result
                0x00 : success
                other : error
********************************************************************************/
MD_STATUS set_command(uint8_t comcode)
{

    MD_STATUS status;

    g_write_commmand[0] = COMBYTE;      /* set command for LCD          */
    g_write_commmand[1] = comcode;      /* set command instruction      */
/*
    send command instruction to LCD
*/

    status = R_IIC_Master_Send( slaveaddr , (uint8_t *__near)g_write_commmand , 2 );
    if( status == 0)
    {
/*
    wait for transmission end
*/
        do
        {
            status = R_IIC_check_comstate();
        }while(( status & 0x0F ) == 0x01 );

/*
    release IIC  bus
*/

        R_IIC_StopCondition();          /* issue stop condition         */

        wait_time(2);                   /* wait for 40us to next        */

    }
    return(status);

}

/*******************************************************************************
* Function Name: wait_100ms
* Description  : This function wait for 100ms
* Arguments    : none
* Return Value : none
********************************************************************************/
void wait_100ms(void)
{
    g_05s_count = 4;                            /* set remain time to 100ms */
    while ( g_05s_status ==0 )                  /* wait for IT interrupt    */
    {
        STOP();
        NOP();
    }
    g_05s_status = 0;

}

/*******************************************************************************
* Function Name: wait_500ms
* Description  : This function wait for 500ms
* Arguments    : none
* Return Value : none
********************************************************************************/
void wait_500ms(void)
{
    g_05s_status = 0;
    while ( g_05s_status == 0 )                 /* wait for IT interrupt    */
    {
        STOP();
        NOP();
    }

    g_05s_status = 0;

}

/********************************************************************************
* Function Name: r_it_interrupt
* Description  : This function is 0.1sec interrupt service routine.
* Arguments    : None
* Return Value : None
*********************************************************************************/
__interrupt static void r_it_interrupt(void)
{

    WDTE = 0xAC;                        /* clear WDT                */
    g_05s_count++;
    if( g_05s_count == 5 )
    {
        g_05s_status = 1;               /* set 0.5s flag            */
        g_05s_count = 0;
    }

}

/* End user code. Do not edit comment generated here */
